import {LOCATION_CHANGE} from './location';

/**
 * 创建方法
 * @param method
 * @param types
 * @param prefix
 * @returns {Function}
 */
function createMenthod(method, types, prefix) {
  return function ({params, data} = {}) {
    return {
      types: types,
      promise: (client) => client[method](prefix, {params, data})
    };
  }
}
/**
 * 创建方法和常量
 * @param prefix  前缀
 * @param actions
 * @param action
 * @param constants
 * @param methods
 * @param name
 * @param pageName
 */
function createMenthodsAndConstants(prefix, actions, action, constants, methods, name, pageName) {
  pageName = pageName ? '-' + pageName : '';

  if (~actions.indexOf(action)) {
    let types = [
      `${prefix}${pageName}/${name}`,
      `${prefix}${pageName}/${name}_SUCCESS`,
      `${prefix}${pageName}/${name}_FAIL`,
      `${prefix}${pageName}/${name}_CLEAR`
    ];
    // 常量
    Object.assign(constants, {
      [name]: types[0],
      [name + '_SUCCESS']: types[1],
      [name + '_FAIL']: types[2],
      [name + '_CLEAR']: types[3]
    });
    // 方法
    if (action === 'C') {
      methods.create = createMenthod('post', types, prefix);
    } else if (action === 'U') {
      methods.update = createMenthod('put', types, prefix);
    } else if (action === 'R') {
      methods.load = createMenthod('get', types, prefix);
    } else if (action === 'D') {
      methods.del = createMenthod('del', types, prefix);
    }
  }
}
/**
 *
 * @param prefix 前缀
 * @param actions
 * @param pageName
 * @returns {{methods: {}, createReducer: createReducer}}
 */
export default function createCURD(prefix, actions, pageName) {
  const constants = {};
  const methods = {};
  const actionsMap = {
    'C': 'CREATE',
    'U': 'UPDATE',
    'R': 'LOAD',
    'D': 'DELETE'
  };

  actions.toUpperCase().split('').forEach((action) => {
    createMenthodsAndConstants(prefix, actions, action, constants, methods, actionsMap[action], pageName);
  });

  const createReducer = function (state, action) {
    switch (action.type) {
      //加载数据中
      case constants.LOAD:
      case constants.CREATE:
      case constants.UPDATE:
      case constants.DELETE:
        return {
          ...state,
          loading: true,
          loaded:false
        };
      //加载成功
      case constants.LOAD_SUCCESS:
      case constants.CREATE_SUCCESS:
      case constants.UPDATE_SUCCESS:
      case constants.DELETE_SUCCESS:
        return {
          ...state,
          loading: false,
          loaded: true,
          data: action.result
        };
      //加载失败
      case constants.LOAD_FAIL:
      case constants.CREATE_FAIL:
      case constants.UPDATE_FAIL:
      case constants.DELETE_FAIL:
        return {
          ...state,
          loading: false,
          loaded: false,
          error: action.error
        };
      // 解决跳出页面 数据还在的问题
      case LOCATION_CHANGE:
            return {}
    }
  };
  return {methods, createReducer};
}
